/*
    keygenerationjob.h

    This file is part of libkleopatra, the KDE keymanagement library
    Copyright (c) 2004 Klarlvdalens Datakonsult AB

    Libkleopatra is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.

    Libkleopatra is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
    the TQt library by Trolltech AS, Norway (or with modified versions
    of TQt that use the same license as TQt), and distribute linked
    combinations including the two.  You must obey the GNU General
    Public License in all respects for all of the code used other than
    TQt.  If you modify this file, you may extend this exception to
    your version of the file, but you are not obligated to do so.  If
    you do not wish to do so, delete this exception statement from
    your version.
*/

#ifndef __KLEO_KEYGENERATIONJOB_H__
#define __KLEO_KEYGENERATIONJOB_H__

#include "job.h"

#include <tqcstring.h>

namespace GpgME {
  class Error;
  class KeyGenerationResult;
}


namespace Kleo {

  /**
     @short An abstract base class for asynchronous key generation

     To use a KeyGenerationJob, first obtain an instance from the
     CryptoBackend implementation, connect the progress() and result()
     signals to suitable slots and then start the key generation with
     a call to start(). This call might fail, in which case the
     KeyGenerationJob instance will have scheduled it's own
     destruction with a call to TQObject::deleteLater().

     After result() is emitted, the KeyGenerationJob will schedule it's own
     destruction by calling TQObject::deleteLater().
  */
  class KeyGenerationJob : public Job {
    Q_OBJECT
  
  protected:
    KeyGenerationJob( TQObject * parent, const char * name );
  public:
    ~KeyGenerationJob();

    /**
       Starts the key generation operation. \a parameters is a
       backend-specific string containing the paramaters of the key to
       create (length, capabilities, etc).
    */
    virtual GpgME::Error start( const TQString & parameters ) = 0;

  signals:
    void result( const GpgME::KeyGenerationResult & result, const TQByteArray & pubKeyData );
  };

}

#endif // __KLEO_KEYGENERATIONJOB_H__
