/*
    cryptoconfigdialog.h

    This file is part of kgpgcertmanager
    Copyright (c) 2004 Klarlvdalens Datakonsult AB

    Libkleopatra is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License,
    version 2, as published by the Free Software Foundation.

    Libkleopatra is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
    the TQt library by Trolltech AS, Norway (or with modified versions
    of TQt that use the same license as TQt), and distribute linked
    combinations including the two.  You must obey the GNU General
    Public License in all respects for all of the code used other than
    TQt.  If you modify this file, you may extend this exception to
    your version of the file, but you are not obligated to do so.  If
    you do not wish to do so, delete this exception statement from
    your version.
*/

#include "cryptoconfigdialog.h"
#include "cryptoconfigmodule.h"
#include <tdelocale.h>
#include <tdeaccelmanager.h>

Kleo::CryptoConfigDialog::CryptoConfigDialog( Kleo::CryptoConfig* config, TQWidget *parent, const char* name )
  : KDialogBase( Swallow,
                 // Remove the "whats's this" button since we have no support for it
                 WStyle_Customize | WStyle_DialogBorder | WStyle_Maximize | WStyle_Title | WStyle_SysMenu,
                 parent, name, true /*modal*/,
                 i18n( "Configure" ), Default|Cancel|Apply|Ok|User1,
                 Ok, true /*separator*/, KGuiItem( i18n( "&Reset" ), "edit-undo" ) )
{
  mMainWidget = new CryptoConfigModule( config, this );
  setMainWidget( mMainWidget );
  connect( mMainWidget, TQT_SIGNAL( changed() ), TQT_SLOT( slotChanged() ) );
  enableButton( Apply, false );
  if ( mMainWidget->hasError() ) {
      showButton( Default, false );
      showButton( User1, false );
      showButton( Apply, false );
      showButton( Ok, false );
  }

  // Automatically assign accelerators
  TDEAcceleratorManager::manage( this );
}

void Kleo::CryptoConfigDialog::slotOk()
{
  slotApply();
  accept();
}

void Kleo::CryptoConfigDialog::slotCancel()
{
  mMainWidget->cancel();
  reject();
}

void Kleo::CryptoConfigDialog::slotDefault()
{
  mMainWidget->defaults();
  slotChanged();
}

void Kleo::CryptoConfigDialog::slotApply()
{
  mMainWidget->save();
  enableButton( Apply, false );
}

void Kleo::CryptoConfigDialog::slotUser1() // reset
{
  mMainWidget->reset();
  enableButton( Apply, false );
}

void Kleo::CryptoConfigDialog::slotChanged()
{
  enableButton( Apply, true );
}

#include "cryptoconfigdialog.moc"
