/*
    This file is part of Kung.

    Copyright (c) 2005 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <schema/simpletype.h>

#include "inputfieldfactory.h"

#include "simplebaseinputfield.h"

SimpleBaseInputField::SimpleBaseInputField( const TQString &name, const Schema::SimpleType *type )
  : SimpleInputField( name, type )
{
  if ( type->subType() == Schema::SimpleType::TypeRestriction ) {
    InputField *field = InputFieldFactory::self()->createBasicField( name, type->baseTypeName(), type );
    if ( !field ) {
      tqDebug( "SimpleBaseInputField: Unable to create field of type %s", type->baseTypeName().latin1() );
    } else {
      appendChild( field );
    }
  } else {
    tqDebug( "SimpleBaseInputField: Unsupported subtype" );
  }
}

void SimpleBaseInputField::setXMLData( const TQDomElement &element )
{
  if ( mName != element.tagName() ) {
    tqDebug( "SimpleBaseInputField: Wrong dom element passed: expected %s, got %s", mName.latin1(), element.tagName().latin1() );
    return;
  }

  if ( mType->subType() == Schema::SimpleType::TypeRestriction ) {
    InputField *field = childField( element.tagName() );
    if ( !field ) {
      tqDebug( "SimpleBaseInputField: Child field %s does not exists", element.tagName().latin1() );
    } else {
      field->setXMLData( element );
    }
  } else {
    tqDebug( "SimpleBaseInputField: Unsupported subtype" );
  }
}

void SimpleBaseInputField::xmlData( TQDomDocument &document, TQDomElement &parent )
{
  if ( mType->subType() == Schema::SimpleType::TypeRestriction ) {
    InputField *field = mFields.first();
    if ( !field ) {
      tqDebug( "SimpleBaseInputField: No child found" );
    } else {
      field->xmlData( document, parent );
    }
  } else {
    tqDebug( "SimpleBaseInputField: Unsupported subtype" );
  }
}

void SimpleBaseInputField::setData( const TQString &data )
{
  if ( mType->subType() == Schema::SimpleType::TypeRestriction ) {
    InputField *field = mFields.first();
    if ( !field ) {
      tqDebug( "SimpleBaseInputField: No child found" );
    } else {
      field->setData( data );
    }
  } else {
    tqDebug( "SimpleBaseInputField: Unsupported subtype" );
  }
}

TQString SimpleBaseInputField::data() const
{
  if ( mType->subType() == Schema::SimpleType::TypeRestriction ) {
    InputField *field = mFields.first();
    if ( !field ) {
      tqDebug( "SimpleBaseInputField: No child found" );
    } else {
      field->data();
    }
  } else {
    tqDebug( "SimpleBaseInputField: Unsupported subtype" );
  }

  return TQString();
}

TQWidget *SimpleBaseInputField::createWidget( TQWidget *parent )
{
  if ( mType->subType() == Schema::SimpleType::TypeRestriction ) {
    InputField *field = mFields.first();
    if ( !field ) {
      tqDebug( "SimpleBaseInputField: No child found" );
      return 0;
    } else {
      return field->createWidget( parent );
    }
  } else {
    tqDebug( "SimpleBaseInputField: Unsupported subtype" );
    return 0;
  }
}
