/*
    This file is part of Kontact.
    Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

    As a special exception, permission is given to link this program
    with any edition of TQt, and distribute the resulting executable,
    without including the source code for TQt in the source distribution.
*/

#ifndef KCMKONTACTKNT_H
#define KCMKONTACTKNT_H

#include <tdecmodule.h>

class TQListViewItem;
class TQSpinxBox;

class TDEAboutData;
class TDEListView;
class KPushButton;

class NewsItem;

class KCMKontactKNT : public TDECModule
{
  Q_OBJECT
  

  public:
    KCMKontactKNT( TQWidget *parent = 0, const char *name = 0 );

    virtual void load();
    virtual void save();
    virtual void defaults();
    virtual const TDEAboutData* aboutData() const;

  private slots:
    void addNews();
    void removeNews();
    void newFeed();
    void deleteFeed();

    void selectedChanged( TQListViewItem *item );
    void allCurrentChanged( TQListViewItem *item );

    void modified();

  private:
    void initGUI();
    void loadNews();
    void loadCustomNews();
    void storeCustomNews();
    void scanNews();

    bool dcopActive() const;

    TDEListView *mAllNews;
    TDEListView *mSelectedNews;
    TQListViewItem *mCustomItem;

    KPushButton *mAddButton;
    KPushButton *mRemoveButton;
    KPushButton *mNewButton;
    KPushButton *mDeleteButton;
    TQSpinBox *mUpdateInterval;
    TQSpinBox *mArticleCount;

    TQMap<TQString, TQString> mFeedMap;
    TQValueList<NewsItem*> mCustomFeeds;
};

class NewsEditDialog : public KDialogBase
{
  Q_OBJECT
  

  public:
    NewsEditDialog( const TQString&, const TQString&, TQWidget *parent );
    TQString title() const;
    TQString url() const;

  private slots:
    void modified();

  private:
    TQLineEdit *mTitle;
    TQLineEdit *mURL;
};

#endif
