/*
    This file is part of the KOrganizer alarm client.

    Copyright (c) 2003 Cornelius Schumacher <schumacher@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

    As a special exception, permission is given to link this program
    with any edition of TQt, and distribute the resulting executable,
    without including the source code for TQt in the source distribution.
*/

#include <stdlib.h>

#include <tdeglobal.h>
#include <kdebug.h>
#include <tdelocale.h>
#include <tdecmdlineargs.h>
#include <tdeaboutdata.h>
#include <kuniqueapplication.h>

#include "koalarmclient.h"

class MyApp : public KUniqueApplication
{
  public:
    MyApp() : mClient( 0 ) {}
    int newInstance()
    {
      // Check if we already have a running alarm daemon widget
      if ( mClient ) return 0;

      mClient = new KOAlarmClient;

      return 0;
    }

  private:
    KOAlarmClient *mClient;
};


static const char korgacVersion[] = "0.9";

static const TDECmdLineOptions options[] =
{
  { 0, 0, 0 }
};

int main( int argc, char **argv )
{
  TDELocale::setMainCatalogue( "korganizer" );
  TDEAboutData aboutData( "korgac", I18N_NOOP("KOrganizer Reminder Daemon"),
                        korgacVersion, I18N_NOOP("KOrganizer Reminder Daemon"),
                        TDEAboutData::License_GPL,
                        "(c) 2003 Cornelius Schumacher",
                        0, "http://pim.kde.org" );
  aboutData.addAuthor( "Cornelius Schumacher", I18N_NOOP("Maintainer"),
                       "schumacher@kde.org" );
  aboutData.addAuthor( "Reinhold Kainhofer", I18N_NOOP("Maintainer"),
                       "kainhofer@kde.org" );

  TDECmdLineArgs::init( argc, argv, &aboutData );
  TDECmdLineArgs::addCmdLineOptions( options );
  KUniqueApplication::addCmdLineOptions();

  if ( !MyApp::start() ) exit( 0 );

  MyApp app;
  app.disableSessionManagement();
  TDEGlobal::locale()->insertCatalogue( "libkcal" );

  return app.exec();
}
