/*
    This file is part of KOrganizer
    Copyright (c) 2004  Bo Thorsen <bo@sonofthor.dk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    In addition, as a special exception, the copyright holders give
    permission to link the code of this program with any edition of
    the TQt library by Trolltech AS, Norway (or with modified versions
    of TQt that use the same license as TQt), and distribute linked
    combinations including the two.  You must obey the GNU General
    Public License in all respects for all of the code used other than
    TQt.  If you modify this file, you may extend this exception to
    your version of the file, but you are not obligated to do so.  If
    you do not wish to do so, delete this exception statement from
    your version.
 */

#ifndef KORGANIZER_SHARED_H
#define KORGANIZER_SHARED_H

#include "korganizeriface.h"

#include <tqobject.h>
#include <kdemacros.h>

class ActionManager;


class KDE_EXPORT KOrganizerIfaceImpl : public TQObject, virtual public KOrganizerIface {
public:
  KOrganizerIfaceImpl( ActionManager* mActionManager,
                       TQObject* parent=0, const char* name=0 );
  ~KOrganizerIfaceImpl();

  bool openURL( const TQString &url );
  bool mergeURL( const TQString &url );
  void closeURL();
  bool saveURL();
  bool canQueryClose();
  bool saveAsURL( const TQString &url );
  TQString getCurrentURLasString() const;
  void syncAllResources();

  bool editIncidence( const TQString &uid );
  bool editIncidence( const TQString &uid, const TQDate &date );

  /** @reimp from KOrganizerIface::deleteIncidence() */
  bool deleteIncidence( const TQString &uid )  { return deleteIncidence( uid, false ); }
  /** @reimp from KOrganizerIface::deleteIncidence() */
  bool deleteIncidence( const TQString &uid, bool force );

  /** @reimp from KOrganizerIface::addIncidence() */
  bool addIncidence( const TQString &iCal );

  /** @reimp from KOrganizerIface::loadProfile() */
  void loadProfile( const TQString& path );

  /** @reimp from KOrganizerIface::saveToProfile() */
  void saveToProfile( const TQString& path ) const;

  /** @reimp from KOrganizerIface::handleCommandLine() */
  bool handleCommandLine();

private:
  ActionManager* mActionManager;
};


#endif // KORGANIZER_SHARED_H

